LWMinimap = {}

function ZO_WorldMapTitleBar_OnDragStart()
    ZO_WorldMap:SetMovable(true)
    ZO_WorldMap:StartMoving()
end

function ZO_WorldMapTitleBar_OnMouseUp(button, upInside)
    ZO_WorldMap:SetMovable(false)
    LWMinimap.settings.anchor = {ZO_WorldMap:GetAnchor(0)}
end

function LWMinimap.UnpackAnchor(anchor)
    anchor[3] = nil
    return unpack(anchor, 2)
end

function LWMinimap.ShowMinimap()  
    if(SetMapToPlayerLocation() == SET_MAP_RESULT_MAP_CHANGED) then
        CALLBACK_MANAGER:FireCallbacks("OnWorldMapChanged")
    end 

    ZO_WorldMap:SetDimensions(unpack(LWMinimap.settings.size))
    ZO_WorldMapScroll:SetDimensions(ZO_WorldMap:GetDimensions())

    if LWMinimap.settings.anchor then
        ZO_WorldMap:ClearAnchors()
        ZO_WorldMap:SetAnchor( LWMinimap.UnpackAnchor(LWMinimap.settings.anchor) )
    end
    ZO_WorldMapZoom:SetHidden(true)
    ZO_WorldMapZoomKeybind:SetHidden(true)
    ZO_WorldMap:SetHidden(false)
    ZO_WorldMapTitle:SetHidden(true)
    ZO_WorldMapTitle:ClearAnchors()
    ZO_WorldMapTitle:SetAnchor(TOP, nil, TOP, 0, -30)
    ZO_WorldMapTitleBar:SetHidden(false)
    
    ZO_WorldMapTitleBar:SetMouseEnabled(true)
    ZO_WorldMapTitleBar:ClearAnchors()
    ZO_WorldMapTitleBar:SetAnchor(TOPLEFT, nil, TOPLEFT, 0, -30)
    ZO_WorldMapTitleBar:SetAnchor(TOPRIGHT, nil, TOPRIGHT, 0, -30)
    ZO_WorldMapContainer:SetHandler("OnUpdate", LWMinimap.OnUpdate)

    LWMinimap.SetMapZoom(LWMinimap.settings.zoomout)
    LWMinimap.SetToPlayer(false)
end

function LWMinimap.SetMapZoom(zoom)
    ZO_WorldMap_PanToPlayer()
    for i=1, zoom do
        ZO_WorldMapZoomMinus_OnClicked()
    end
end

function LWMinimap.HideMinimap()
    -- Anything other than 2 is fast travel: wayshrine, keep links, etc.
    if (ZO_WorldMap_GetMode() == MAP_MODE_LARGE_CUSTOM) then
        ZO_WorldMap:SetHidden(true)
    end
    
    ZO_WorldMapTitle:SetHidden(true)
    ZO_WorldMapTitleBar:SetHidden(true)
    ZO_WorldMapTitleBarBG:SetHidden(true)

    ZO_WorldMap:SetDimensions(unpack(LWMinimap.defaultDimensions))
    ZO_WorldMap:ClearAnchors()
    ZO_WorldMap:SetAnchor(LWMinimap.UnpackAnchor(LWMinimap.defaultAnchor))
    ZO_WorldMapScroll:SetDimensions(ZO_WorldMap:GetDimensions())
    ZO_WorldMapZoomKeybind:SetHidden(false)
    ZO_WorldMapContainer:SetHandler("OnUpdate", LWMinimap.defaultOnUpdate)
    LWMinimap.SetMapZoom(LWMinimap.settings.zoomoutdefault)
end

function LWMinimap.SetToPlayer(forced)
    local x, y = GetMapPlayerPosition("player")
    if forced or (LWMinimap.lastX == x and LWMinimap.lastY == y) then
        return
    end
    
    LWMinimap.lastX = x
    LWMinimap.lastY = y
    local width, height = ZO_WorldMapContainer:GetDimensions()
    local frameWidth, frameHeight = ZO_WorldMapScroll:GetDimensions()
    x = width * x
    y = height * y
    x = zo_min( zo_max(frameWidth / 2, x), width - frameWidth / 2)
    y = zo_min( zo_max(frameHeight / 2, y), height - frameHeight / 2)
    ZO_WorldMapContainer:ClearAnchors()
    ZO_WorldMapContainer:SetAnchor(TOPLEFT, nil, CENTER, -x, -y)
end
-- Clean and dry up the settings methods later.
function LWMinimap.SetSize(arg)
    arg = tonumber(arg)
    if arg == nil or arg < 200 or arg > 800 then
        d("/lwmsize [size]")
        d("[size] between 200 and 800.")
        return
    end
    
    LWMinimap.settings.size = { arg, arg }
    LWMinimap.ShowMinimap()
end

function LWMinimap.SetZoomOut(arg)
    arg = tonumber(arg)
    if arg == nil or arg < 0 or arg > 8 then
        d("Sets the zoom out level of the mini map.")
        d("/lwmzoom [zoomout clicks]")
        d("[zoomout clicks] between 0 and 8.")
        return
    end
    LWMinimap.settings.zoomout = arg
    LWMinimap.SetMapZoom(LWMinimap.settings.zoomout)
end

function LWMinimap.SetDefaultZoomOut(arg)
    arg = tonumber(arg)
    if arg == nil or arg < 0 or arg > 8 then
        d("Sets the zoom out level of the default map.")
        d("/lwmdefaultzoom [zoomout clicks]")
        d("Where [zoomout clicks] is between 0 and 8.")
        return
    end
    LWMinimap.settings.zoomoutdefault = arg
end

function LWMinimap.OnUpdate(time)
    LWMinimap.SetToPlayer()
end

function LWMinimap.OnPlayerActivated()
    LWMinimap.init = true
    LWMinimap.ShowMinimap()
    EVENT_MANAGER:UnregisterForEvent("LightWeightMinimap", EVENT_PLAYER_ACTIVATED)
end

function LWMinimap.OnLoad(eventCode, addOnName)
    if addOnName ~= "LightWeightMinimap" then
        return
    end
    SLASH_COMMANDS["/lwm"] = function() d('/lwmsize, /lwmzoom, /lwmdefaultzoom') end
    SLASH_COMMANDS["/lwmsize"] = LWMinimap.SetSize
    SLASH_COMMANDS["/lwmzoom"] = LWMinimap.SetZoomOut
    SLASH_COMMANDS["/lwmdefaultzoom"] = LWMinimap.SetDefaultZoomOut
    LWMinimap.modeStack = {}
    LWMinimap.defaultDimensions = {ZO_WorldMap:GetDimensions()}
    LWMinimap.defaultAnchor = {ZO_WorldMap:GetAnchor(0)}
    LWMinimap.defaultOnUpdate = ZO_WorldMapContainer:GetHandler("OnUpdate")
    
    LWMinimap.settings = ZO_SavedVars:NewAccountWide("LWMinimap_SavedVars", 2, "settings", 
        { size = {200, 200}, 
        anchor = {
                        [2] = 9,
                        [3] = nil,
                        [4] = 9,
                        [5] = -10,
                        [6] = 30,
                },
        zoomout = 3,
        zoomoutdefault = 5})

    local tlw
    tlw = WINDOW_MANAGER:CreateTopLevelWindow()
    tlw:SetDimensions(1,1)
    tlw:SetAnchor(CENTER, GuiRoot, CENTER, 0, 0)
    tlw:SetHidden(true)
 
    local fragment = ZO_SimpleSceneFragment:New(tlw)
    fragment:RegisterCallback("StateChange", LWMinimap.OnStateChange)

    local sceneHud = SCENE_MANAGER:GetScene("hud")
    local sceneHudUI = SCENE_MANAGER:GetScene("hudui")
    sceneHud:AddFragment(fragment)
    sceneHudUI:AddFragment(fragment)
    -- Event for player loading into game
    EVENT_MANAGER:RegisterForEvent("LightWeightMinimap", EVENT_PLAYER_ACTIVATED , LWMinimap.OnPlayerActivated)
    -- ZO_WorldMap is resized after a screen resize. Catch the event and reset the minimap.
    EVENT_MANAGER:RegisterForEvent("LightWeightMinimap", EVENT_SCREEN_RESIZED, LWMinimap.OnScreenResized)
    -- Pressing 'm' from a fast travel map closes the ui.
    EVENT_MANAGER:RegisterForEvent("LightWeightMinimap", EVENT_END_FAST_TRAVEL_INTERACTION, LWMinimap.ShowMinimap)
    -- Zoom out the default world map when it is opened.
    local worldMap = SCENE_MANAGER:GetScene("worldMap")
    worldMap:RegisterCallback("StateChange", LWMinimap.OnWorlMapStateChange)
end

function LWMinimap.OnScreenResized(x, y)
    if SCENE_MANAGER:GetScene("hud").state == SCENE_SHOWN or SCENE_MANAGER:GetScene("hudui").state == SCENE_SHOWN then
        LWMinimap.ShowMinimap()
    end
end

function LWMinimap.OnStateChange(oldState, newState)
    if(newState == SCENE_SHOWN) then
        LWMinimap.ShowMinimap()
    elseif(newState == SCENE_HIDING) then
        LWMinimap.HideMinimap()
    end
end

function LWMinimap.OnWorlMapStateChange(oldState, newState)
    if(newState == SCENE_SHOWN) then
        LWMinimap.SetMapZoom(LWMinimap.settings.zoomoutdefault)
    end
end

EVENT_MANAGER:RegisterForEvent("LightWeightMinimap", EVENT_ADD_ON_LOADED, LWMinimap.OnLoad)
